'usefull for data structures

TYPE CVector4
SINGLE		.x
SINGLE		.y
SINGLE		.z
SINGLE		.w
END TYPE

TYPE CVector3
SINGLE		.x
SINGLE		.y
SINGLE		.z
END TYPE

'natives MD3 data structures

TYPE MD3Header
STRING*4							.strID
XLONG									.version
STRING*64							.strName
XLONG									.flags
XLONG									.nFrames
XLONG									.nTags
XLONG									.nSurfaces
XLONG									.nSkins
XLONG									.oFrames
XLONG									.oTags
XLONG									.oSurfaces
XLONG									.oEOF
END TYPE	

TYPE MD3Surface
STRING*4							.strID
STRING*64							.strName
XLONG									.flags
XLONG									.nFrames
XLONG									.nShaders
XLONG									.nVerts
XLONG									.nTriangles
XLONG									.oTriangles
XLONG									.oShaders
XLONG									.oTxCoords
XLONG									.oVertices
XLONG									.oEND
END TYPE

TYPE MD3Frame
CVector3							.boxmin
CVector3							.boxmax
CVector3							.origin
SINGLE								.radius
STRING*16							.strName
END TYPE

TYPE MD3Shader
STRING*64							.strName
XLONG									.shaderIndex
END TYPE

TYPE MD3Tag
STRING*64							.strName
CVector3							.origin
SINGLE								.rotmatrix[8]
END TYPE


'internals MD3 data strutures 

$$MAX_MD3 = 256
$$MAX_SURFACES = 32
$$MAX_FRAMES = 1024
$$MAX_TAGS=16

$$MD3_LOADED = 1

$$MD3_DONOTUPDATE = 1

'instance surface flags
$$MD3_SURFACE_FLAG_IGNORE = 1
$$MD3_SURFACE_FLAG_NO_SHADOW = 2

'Version : DLL version info must match with DEC version

$$MD3_VERSION = 1


TYPE tMD3Surface
XLONG									.nTriangles
XLONG									.shaderID
XLONG									.sFrame						'frame size for Verts so offset = frame*surface.sVerts
XLONG									.pTriangles				'pointers to data
XLONG									.pTxCoords
XLONG									.pVerts
STRING*64							.strName
END TYPE

TYPE tMD3Tag
CVector3							.origin
CVector4							.quat
END TYPE	

TYPE tMD3Frame
MD3Frame							.frame
tMD3Tag								.tag[$$MAX_TAGS]
END TYPE

TYPE tMD3ModelFrames
tMD3Frame							.cFrame[$$MAX_FRAMES]
END TYPE

TYPE tMD3Model
STRING*256						.strName
XLONG									.flags
XLONG									.nFrames
XLONG									.nTags
XLONG									.nSurfaces
tMD3Surface						.surface[$$MAX_SURFACES]
STRING*64							.strTagName[$$MAX_TAGS]
END	TYPE	

TYPE tMD3instance
XLONG									.model
XLONG									.nSurfaces
XLONG									.surfaceflags[$$MAX_SURFACES]	'used to control surface behavior
XLONG									.pVerts[$$MAX_SURFACES] 'MD3 instances use only one frame and only vertices change every frames
XLONG									.shaderID[$$MAX_SURFACES] 'you can change shader for any instance

XLONG									.pShadowFrontTris[$$MAX_SURFACES] 'Frontcap tris
XLONG									.nShadowTris[$$MAX_SURFACES]			'number of shadow tris

XLONG									.pShadowEdges[$$MAX_SURFACES]			'shadow edges, extruded to build shadows
XLONG									.nShadowEdges[$$MAX_SURFACES]			'edges 3XLONG==> vertexindex1 , ~2,count

CVector3							.vLpos		'Light pos for shadow generation (usefull for projection and debug purpose)								

tMD3Tag								.tag[$$MAX_TAGS] 'idem for tags
CVector3							.vPos
CVector3							.boxmin 'the box aligned
CVector3							.boxmax
CVector3							.origin
CVector3							.fp1			'the points of the box : front
CVector3							.fp2
CVector3							.fp3
CVector3							.fp4
CVector3							.bp1			'backpoints
CVector3							.bp2
CVector3							.bp3
CVector3							.bp4

CVector3							.oView				'view vector for weapon/cam
CVector3							.dView
XLONG									.ownVV
SINGLE								.iMatrix[15]		 'instance matrix

'Instance interface
FUNCADDR							.Destroy ()
FUNCADDR							.BuildStatic (XLONG)
FUNCADDR							.BuildAnim (XLONG,XLONG,SINGLE,XLONG)
FUNCADDR							.SetIdentity (SINGLE,SINGLE,SINGLE,CVector3)
FUNCADDR							.GetTagID (STRING)
FUNCADDR							.Link (tMD3instance,XLONG,CVector3)
FUNCADDR							.ShowBox ()
FUNCADDR							.GetSurfaceID (STRING)
FUNCADDR							.SetSurfaceShader (XLONG,XLONG)
FUNCADDR							.SetViewVector (CVector3,CVector3)
FUNCADDR							.BuildRender ()
FUNCADDR							.BuildBox ()
FUNCADDR							.SetSurfaceFlags (XLONG,XLONG)

XLONG									.bDestroy[16]
XLONG									.bBuildStatic[16]
XLONG									.bBuildAnim[16]
XLONG									.bSetIdentity[16]
XLONG									.bGetTagID[16]
XLONG									.bLink[16]
XLONG									.bShowBox[16]
XLONG									.bGetSurfaceID[16]
XLONG									.bSetSurfaceShader[16]
XLONG									.bSetViewVector[16]
XLONG									.bBuildRender[16]
XLONG									.bBuildBox[16]
XLONG									.bSetSurfaceFlags[16]

END TYPE

TYPE MD3_INFOS
XLONG									.LastModel
XLONG									.Maxmodels
XLONG									.flags

FUNCADDR							.fOpen(STRING,XLONG)
FUNCADDR							.fRead(XLONG,XLONG,XLONG)
FUNCADDR							.fClose(XLONG)
FUNCADDR							.fSeek(XLONG,XLONG)
FUNCADDR							.Print(STRING)
FUNCADDR							.mAlloc(XLONG)
FUNCADDR							.mFree(XLONG)
FUNCADDR							.SHADER_Load(STRING)
FUNCADDR							.SHADER_Free(XLONG)
FUNCADDR							.PushSurface(XLONG,XLONG,XLONG,XLONG)

XLONG									.DllHdl
FUNCADDR							.GetCPUID()
FUNCADDR							.CopyMemory(XLONG,XLONG,XLONG,XLONG)
FUNCADDR							.CopyMemoryMMX(XLONG,XLONG,XLONG,XLONG,XLONG)
FUNCADDR							.FloatInterpol(XLONG,XLONG,XLONG,XLONG,SINGLE)

END TYPE       

'future usage : stencil shadows
$$MD3_USE_SHADOWS = 1
'I/O
$$MD3_OPTION_FOPEN = 1
$$MD3_OPTION_FREAD = 2
$$MD3_OPTION_FCLOSE = 3
$$MD3_OPTION_FSEEK = 4
$$MD3_OPTION_PRINT = 5
'Memory management
$$MD3_OPTION_MALLOC = 6
$$MD3_OPTION_MFREE = 7
'Shaders
$$MD3_OPTION_SHADER_LOAD = 8
$$MD3_OPTION_SHADER_FREE = 9
'Render
$$MD3_OPTION_PUSH_SURFACE = 10

'Default method render flags
$$MD3_DEF_LIGHT=2
$$MD3_DEF_TEXTURE=1
